<?php
/**
 * Add ARIA and svg icons to links
 *
 * The following type of links filtered:
 * 	PDF, external links, image in links. 
 *
 * Theme: Manduca - focus on accessibility
 * @ Since 17.9.6
 * */

 Class Link_Functions {
   
   function add_hook_to_wp() {
		// filters have high (late) priority to make sure that any markup plugins have done their HTML. 
		add_filter( 'the_content', array( $this , 'filter_links' ) , 999 );
		add_filter( 'the_excerpt', array( $this , 'filter_links' ) , 999 );
		add_filter( 'comment_text', array( $this , 'filter_links' ) , 999 );
		add_filter( 'widget_text', array( $this , 'filter_links' ) , 999 );

	}
	
	/*
	 * Filters the <a> elements and add accessibility elements
	 *
	 * @param $text : string text to filter
	 * 	match information:
	 * 		0: full link
	 * 		1: attributes befor href
	 * 		2: protocol
	 * 		3: link uri
	 * 		4: attributes after href
	 * 		5: link title
	 **/
	function filter_links( $text ) {
		$pattern = '/<a (.*?)href="(.*?)\/?\/?(.*?)"(.*?)>(.*?)<\/a>/i';
		$text = preg_replace_callback( $pattern, array( $this, 'parse_links' ) , $text );
		return $text;
	}

	
	
	/**
	 * Callback function of preg_replace to parse links
	 *
	 * @param $matches HTML code to filter
	 *
	 * @return modified HTML code, where class="ext-links" are added. 
	 **/
	function parse_links( $matches ) {
	  /* Since WP 4.7.4 duplicate callback from apply_filters is possible.
	   * This condition returns if link inlcudes svg.
	   * since 18.1
	   * */
	  if( strpos( $matches[5] , 'svg' ) ) {
			return  $this->construct_anchor( $matches, '' );
	  }

		if ( ! isset( $matches[2][0] )  ) {
			$anchor_link = true;
			
		}
		elseif( $matches[2][0] !=='#') {
			$anchor_link = false;
		}
		else {
			$anchor_link = false;
		}
		
		$home_domain = $this->get_domain_name_from_uri( $_SERVER["HTTP_HOST"] );
		$link_domain = $this->get_domain_name_from_uri( $matches[3] );	  
		if( !$link_domain) {			  
			  $link_domain = $home_domain;			  
		}
		
		/*
		 *External link
		 **/
		if(  $link_domain !=  $home_domain &&  $anchor_link === false ) {
			
			//Check if google-map
			if( false !== strpos( $matches[3], 'goo.gl/maps' ) || false !== strpos( $matches[3], 'google.hu/maps' ) ) {
			
				  //Translatos: add aria label to links to google maps. 
				  $matches [4] .= 'aria-label="' .__( 'open map', 'manduca' ) .'" ';
				  $svg_icon 	= manduca_get_svg( array( 'icon' => 'map-marker') ) ;
			}
			else {
				  //Translators: add aria-label to link. 
				  $matches [4] .= 'aria-label="' .__( 'external', 'manduca' ) .'" ';
				  $svg_icon 	=manduca_get_svg( array( 'icon' => 'extlink') ) ;
			}
			
			$class	= 'ext-link';
			if( false != strpos( $matches [4], 'target="_blank"' ) )  {
				  $class .=' new-window';
				  
				  //Translators: add screen-reader text to link
				  $matches[5] .= sprintf( '<span>%s</span>', __( 'Opens a new window' , 'manduca' ) ) ;
			}
			$matches = $this->append_class_to_html( $matches , $class );
			
			$pattern = $this->construct_anchor( $matches,  $svg_icon );
			
		
		}
		
		/*
		 *These are checked if it is not an external link
		 **/
		else {
			
			// Word document	
			if( strpos( $matches[3], 'docx' ) !== false || strpos( $matches[3], 'doc' ) !== false  ) {
				
				$matches = $this->append_class_to_html( $matches , 'media' );
				//translators: Aria label for doc and docx links. 
				$matches [4] .= 'aria-label="' .__( 'MS Word document', 'manduca' ) .'" ';
				$pattern = $this->construct_anchor( $matches, manduca_get_svg( array( 'icon' => 'word') ) );
			
			}
			
			//Excel document
			elseif( strpos( $matches[3], 'xlsx' ) !== false || strpos( $matches[3], 'xls' ) !== false  ) {
			
				$matches = $this->append_class_to_html( $matches , 'media' );
				//translators: Aria label for xls and xlsx links. 
				$matches [4] .= 'aria-label="' .__( 'MS Excel document', 'manduca' ) .'" ';
				$pattern = $this->construct_anchor ( $matches, manduca_get_svg( array( 'icon' => 'excel') ) );
			
			}
			
			//PDF document
			elseif( strpos( $matches[3], 'pdf' ) !== false ) {
			
				$matches = $this->append_class_to_html( $matches , 'media' );
				//translators: Aria label for pdf links. 
				$matches [4] .= 'aria-label="' .__( 'PDF document', 'manduca' ) .'" ';
				$pattern = $this->construct_anchor ( $matches, manduca_get_svg( array( 'icon' => 'pdf') ) );
			
			}
			
			/*
			 * There is nothing to do with the link
			 * re-assemble again. 
			 * */
			else {
			
				$pattern = $this->construct_anchor( $matches, '' );
			}
		}
			
		return $pattern;
	}
	
	
	
	
	/*
	* Get domain name from url 
	*
	* @parameter string $uri URI of to be analized 
	* */
	
	function get_domain_name_from_uri( $uri ) {
		preg_match( '/^(http:\/\/|https:\/\/)[A-Za-z0-9.-]+(?!.*\|\w*$)/i', $uri, $matches );
		if( !isset( $match[2] ) ) {
			  return false;
		}
		$host = $matches[2];
		preg_match( '/[^\.\/]+\.[^\.\/]+$/', $host, $matches );
		return $host;
	}

	


	
	
	
	/**
	 * Append HTMl class to the HTMl tokens
	 *
	 * @param array: tokens of link
	 * @param class: class to be appended.
	 *
	 * @ return array: appended with class
	 * @ since 17.9.7
	 **/
	
	 function append_class_to_html( $tokens, $class ) {
		
			// Check the tokens before href
			if( !empty ($tokens[1]) ) {
				if ( strpos( $tokens[1], 'class=') !== FALSE   ) {
					// Already has calss attribute
					$tokens[1] = str_replace ( 'class="', 'class="' .$class .' ', $tokens[1]);
					return $tokens;
				}
			}
			
			// Check the tokens after href
			if( !empty ($tokens[4]) ) {
				  if ( strpos( $tokens[4], 'class=') !== FALSE   ) {
					  // Already has calss attribute
					  $tokens[4] = str_replace ( 'class="', 'class="' .$class .' ', $tokens[4]);
			  
				  }
				  else {
						$tokens[4] .= ' class="' .$class .'" ';
				  }
			}
			
			return $tokens;
	}
 
 
 
	/**
	 * Construct anchor (<a>) link of media links and adds an icon
	 *
	 * @param array $tokens parts of the anchor link
	 * @param array $tokens parts of the anchor link
	 *
	 * @return HTML element
	 *
	 * @since 17.9.7
	 * @fixed an error of the anchor links 18.4.2
	 **/
		
	function construct_anchor( $tokens, $svg ) {
		if( !empty( $tokens[2] ) ) {
		  $tokens[2] = sprintf( '%s//',
					     $tokens[2]
					     );
		}
		return sprintf ( '<a href="%s%s" %s %s>%s%s</a>',
								   $tokens[2],
								   $tokens[3],
								   $tokens[1],
								   $tokens[4],
								   $tokens[5],
								   $svg
								   );
	}

 }